@@ -20,6 +20,11 @@ if [[ -n "${LLVM_SYMBOLIZER}" ]] && [[ ! $(type -P "${LLVM_SYMBOLIZER}") ]]; the
   exit 1
 fi
 
+echo "before_script.sh: ccache stats (will be cleared)"
+ccache -s
+# Reset ccache stats for real results in before_cache.
+ccache --zero-stats
+
 if [[ "${TRAVIS_OS_NAME}" == osx ]]; then
   # Adds user to a dummy group.
   # That allows to test changing the group of the file by `os_fchown`.
