@@ -34,32 +34,6 @@ int uv_pipe_open(struct uv_pipe_s *handle, int fd)
   return result;
 }
 
-// Issue 2422
-//
-// Teach coverity about jemalloc functions, so that it understands
-// they are equivalent to malloc ones.
-
-void *je_malloc(size_t size)
-{
-  return __coverity_alloc__(size);
-}
-
-void je_free(void *ptr)
-{
-  __coverity_free__(ptr);
-}
-
-void *je_calloc(size_t count, size_t size)
-{
-  return je_malloc(count * size);
-}
-
-void *je_realloc(void *ptr, size_t size)
-{
-  je_free(ptr);
-  return je_malloc(size);
-}
-
 // Hint Coverity that adding item to d avoids losing track
 // of the memory allocated for item.
 typedef struct {} dictitem_T;
