@@ -6,6 +6,7 @@
 #include <string.h>
 
 #include "nvim/func_attr.h"
+#include "nvim/types.h"
 
 #define ARRAY_DICT_INIT {.size = 0, .capacity = 0, .items = NULL}
 #define STRING_INIT {.data = NULL, .size = 0}
@@ -20,8 +21,6 @@
 # define DictionaryOf(...) Dictionary
 #endif
 
-typedef int handle_T;
-
 // Basic types
 typedef enum {
   kErrorTypeNone = -1,
@@ -30,14 +29,13 @@ typedef enum {
 } ErrorType;
 
 typedef enum {
-  kMessageTypeRequest,
-  kMessageTypeResponse,
-  kMessageTypeNotification
+  kMessageTypeUnknown = -1,
+  // Per msgpack-rpc spec.
+  kMessageTypeRequest = 0,
+  kMessageTypeResponse = 1,
+  kMessageTypeNotification = 2,
 } MessageType;
 
-/// Used as the message ID of notifications.
-#define NO_RESPONSE UINT64_MAX
-
 /// Mask for all internal calls
 #define INTERNAL_CALL_MASK (((uint64_t)1) << (sizeof(uint64_t) * 8 - 1))
 
@@ -47,7 +45,7 @@ typedef enum {
 /// Internal call from lua code
 #define LUA_INTERNAL_CALL (VIML_INTERNAL_CALL + 1)
 
-static inline bool is_internal_call(uint64_t channel_id)
+static inline bool is_internal_call(const uint64_t channel_id)
   REAL_FATTR_ALWAYS_INLINE REAL_FATTR_CONST;
 
 /// Check whether call is internal
@@ -106,6 +104,7 @@ typedef enum {
   kObjectTypeString,
   kObjectTypeArray,
   kObjectTypeDictionary,
+  kObjectTypeLuaRef,
   // EXT types, cannot be split or reordered, see #EXT_OBJECT_TYPE_SHIFT
   kObjectTypeBuffer,
   kObjectTypeWindow,
@@ -121,6 +120,7 @@ struct object {
     String string;
     Array array;
     Dictionary dictionary;
+    LuaRef luaref;
   } data;
 };
 
