@@ -14,7 +14,6 @@
 #include "nvim/vim.h"
 #include "nvim/ascii.h"
 #include "nvim/charset.h"
-#include "nvim/farsi.h"
 #include "nvim/func_attr.h"
 #include "nvim/indent.h"
 #include "nvim/main.h"
@@ -112,12 +111,6 @@ int buf_init_chartab(buf_T *buf, int global)
       g_chartab[c++] = 1 + CT_PRINT_CHAR;
     }
 
-    if (p_altkeymap) {
-      while (c < YE) {
-        g_chartab[c++] = 1 + CT_PRINT_CHAR;
-      }
-    }
-
     while (c < 256) {
       if (c >= 0xa0) {
         // UTF-8: bytes 0xa0 - 0xff are printable (latin1)
@@ -173,7 +166,7 @@ int buf_init_chartab(buf_T *buf, int global)
       }
 
       if (ascii_isdigit(*p)) {
-        c = getdigits_int((char_u **)&p);
+        c = getdigits_int((char_u **)&p, true, 0);
       } else {
         c = mb_ptr2char_adv(&p);
       }
@@ -183,7 +176,7 @@ int buf_init_chartab(buf_T *buf, int global)
         ++p;
 
         if (ascii_isdigit(*p)) {
-          c2 = getdigits_int((char_u **)&p);
+          c2 = getdigits_int((char_u **)&p, true, 0);
         } else {
           c2 = mb_ptr2char_adv(&p);
         }
@@ -217,8 +210,7 @@ int buf_init_chartab(buf_T *buf, int global)
         // "C".
         if (!do_isalpha
             || mb_islower(c)
-            || mb_isupper(c)
-            || (p_altkeymap && (F_isalpha(c) || F_isdigit(c)))) {
+            || mb_isupper(c)) {
           if (i == 0) {
             // (re)set ID flag
             if (tilde) {
@@ -230,9 +222,7 @@ int buf_init_chartab(buf_T *buf, int global)
             // (re)set printable
             // For double-byte we keep the cell width, so
             // that we can detect it from the first byte.
-            if (((c < ' ')
-                 || (c > '~')
-                 || (p_altkeymap && (F_isalpha(c) || F_isdigit(c))))) {
+            if (((c < ' ') || (c > '~'))) {
               if (tilde) {
                 g_chartab[c] = (uint8_t)((g_chartab[c] & ~CT_CELL_MASK)
                                          + ((dy_flags & DY_UHEX) ? 4 : 2));
@@ -540,8 +530,7 @@ char_u *transchar(int c)
     c = K_SECOND(c);
   }
 
-  if ((!chartab_initialized && (((c >= ' ') && (c <= '~'))
-                                || (p_altkeymap && F_ischar(c))))
+  if ((!chartab_initialized && (((c >= ' ') && (c <= '~'))))
       || ((c <= 0xFF) && vim_isprintc_strict(c))) {
     // printable character
     transchar_buf[i] = (char_u)c;
@@ -749,8 +738,8 @@ int vim_strnsize(char_u *s, int len)
 ///
 /// @return Number of characters.
 #define RET_WIN_BUF_CHARTABSIZE(wp, buf, p, col) \
-  if (*(p) == TAB && (!(wp)->w_p_list || lcs_tab1)) { \
-    const int ts = (int) (buf)->b_p_ts; \
+  if (*(p) == TAB && (!(wp)->w_p_list || wp->w_p_lcs_chars.tab1)) { \
+    const int ts = (int)(buf)->b_p_ts; \
     return (ts - (int)(col % ts)); \
   } else { \
     return ptr2cells(p); \
@@ -1022,12 +1011,12 @@ int win_lbr_chartabsize(win_T *wp, char_u *line, char_u *s, colnr_T col, int *he
       && vim_isbreak(c)
       && !vim_isbreak((int)s[1])
       && wp->w_p_wrap
-      && (wp->w_width != 0)) {
+      && (wp->w_width_inner != 0)) {
     // Count all characters from first non-blank after a blank up to next
     // non-blank after a blank.
     numberextra = win_col_off(wp);
     col2 = col;
-    colmax = (colnr_T)(wp->w_width - numberextra - col_adj);
+    colmax = (colnr_T)(wp->w_width_inner - numberextra - col_adj);
 
     if (col >= colmax) {
         colmax += col_adj;
@@ -1076,9 +1065,9 @@ int win_lbr_chartabsize(win_T *wp, char_u *line, char_u *s, colnr_T col, int *he
     numberextra = numberwidth;
     col += numberextra + mb_added;
 
-    if (col >= (colnr_T)wp->w_width) {
-      col -= wp->w_width;
-      numberextra = wp->w_width - (numberextra - win_col_off2(wp));
+    if (col >= (colnr_T)wp->w_width_inner) {
+      col -= wp->w_width_inner;
+      numberextra = wp->w_width_inner - (numberextra - win_col_off2(wp));
       if (col >= numberextra && numberextra > 0) {
         col %= numberextra;
       }
@@ -1097,16 +1086,18 @@ int win_lbr_chartabsize(win_T *wp, char_u *line, char_u *s, colnr_T col, int *he
       numberwidth -= win_col_off2(wp);
     }
 
-    if (col == 0 || (col + size + sbrlen > (colnr_T)wp->w_width)) {
+    if (col == 0 || (col + size + sbrlen > (colnr_T)wp->w_width_inner)) {
       added = 0;
 
       if (*p_sbr != NUL) {
-        if (size + sbrlen + numberwidth > (colnr_T)wp->w_width) {
+        if (size + sbrlen + numberwidth > (colnr_T)wp->w_width_inner) {
           // Calculate effective window width.
-          int width = (colnr_T)wp->w_width - sbrlen - numberwidth;
-          int prev_width = col ? ((colnr_T)wp->w_width - (sbrlen + col)) : 0;
-          if (width == 0) {
-            width = (colnr_T)wp->w_width;
+          int width = (colnr_T)wp->w_width_inner - sbrlen - numberwidth;
+          int prev_width = col ? ((colnr_T)wp->w_width_inner - (sbrlen + col))
+                               : 0;
+
+          if (width <= 0) {
+            width = 1;
           }
           added += ((size - prev_width) / width) * vim_strsize(p_sbr);
           if ((size - prev_width) % width) {
@@ -1148,7 +1139,7 @@ static int win_nolbr_chartabsize(win_T *wp, char_u *s, colnr_T col, int *headp)
 {
   int n;
 
-  if ((*s == TAB) && (!wp->w_p_list || lcs_tab1)) {
+  if ((*s == TAB) && (!wp->w_p_list || wp->w_p_lcs_chars.tab1)) {
     n = (int)wp->w_buffer->b_p_ts;
     return n - (col % n);
   }
@@ -1175,11 +1166,11 @@ bool in_win_border(win_T *wp, colnr_T vcol)
   int width1;             // width of first line (after line number)
   int width2;             // width of further lines
 
-  if (wp->w_width == 0) {
+  if (wp->w_width_inner == 0) {
     // there is no border
     return false;
   }
-  width1 = wp->w_width - win_col_off(wp);
+  width1 = wp->w_width_inner - win_col_off(wp);
 
   if ((int)vcol < width1 - 1) {
     return false;
@@ -1240,7 +1231,7 @@ void getvcol(win_T *wp, pos_T *pos, colnr_T *start, colnr_T *cursor,
   // When 'list', 'linebreak', 'showbreak' and 'breakindent' are not set
   // use a simple loop.
   // Also use this when 'list' is set but tabs take their normal size.
-  if ((!wp->w_p_list || (lcs_tab1 != NUL))
+  if ((!wp->w_p_list || (wp->w_p_lcs_chars.tab1 != NUL))
       && !wp->w_p_lbr
       && (*p_sbr == NUL)
       && !wp->w_p_bri ) {
@@ -1604,59 +1595,69 @@ char_u* skiptowhite_esc(char_u *p) {
   return p;
 }
 
-/// Get a number from a string and skip over it, signalling overflows
+/// Gets a number from a string and skips over it, signalling overflow.
 ///
 /// @param[out]  pp  A pointer to a pointer to char_u.
 ///                  It will be advanced past the read number.
 /// @param[out]  nr  Number read from the string.
 ///
-/// @return OK on success, FAIL on error/overflow
-int getdigits_safe(char_u **pp, intmax_t *nr)
+/// @return true on success, false on error/overflow
+bool try_getdigits(char_u **pp, intmax_t *nr)
 {
   errno = 0;
   *nr = strtoimax((char *)(*pp), (char **)pp, 10);
-
-  if ((*nr == INTMAX_MIN || *nr == INTMAX_MAX)
-      && errno == ERANGE) {
-    return FAIL;
+  if (errno == ERANGE && (*nr == INTMAX_MIN || *nr == INTMAX_MAX)) {
+    return false;
   }
-
-  return OK;
+  return true;
 }
 
-/// Get a number from a string and skip over it.
+/// Gets a number from a string and skips over it.
 ///
-/// @param[out]  pp  A pointer to a pointer to char_u.
+/// @param[out]  pp  Pointer to a pointer to char_u.
 ///                  It will be advanced past the read number.
+/// @param strict    Abort on overflow.
+/// @param def       Default value, if parsing fails or overflow occurs.
 ///
-/// @return Number read from the string.
-intmax_t getdigits(char_u **pp)
+/// @return Number read from the string, or `def` on parse failure or overflow.
+intmax_t getdigits(char_u **pp, bool strict, intmax_t def)
 {
   intmax_t number;
-  int ret = getdigits_safe(pp, &number);
-
-  (void)ret;  // Avoid "unused variable" warning in Release build
-  assert(ret == OK);
-
-  return number;
+  int ok = try_getdigits(pp, &number);
+  if (strict && !ok) {
+    abort();
+  }
+  return ok ? number : def;
 }
 
-/// Get an int number from a string. Like getdigits(), but restricted to `int`.
-int getdigits_int(char_u **pp)
+/// Gets an int number from a string.
+///
+/// @see getdigits
+int getdigits_int(char_u **pp, bool strict, int def)
 {
-  intmax_t number = getdigits(pp);
+  intmax_t number = getdigits(pp, strict, def);
 #if SIZEOF_INTMAX_T > SIZEOF_INT
-  assert(number >= INT_MIN && number <= INT_MAX);
+  if (strict) {
+    assert(number >= INT_MIN && number <= INT_MAX);
+  } else if (!(number >= INT_MIN && number <= INT_MAX)) {
+    return def;
+  }
 #endif
   return (int)number;
 }
 
-/// Get a long number from a string. Like getdigits(), but restricted to `long`.
-long getdigits_long(char_u **pp)
+/// Gets a long number from a string.
+///
+/// @see getdigits
+long getdigits_long(char_u **pp, bool strict, long def)
 {
-  intmax_t number = getdigits(pp);
+  intmax_t number = getdigits(pp, strict, def);
 #if SIZEOF_INTMAX_T > SIZEOF_LONG
-  assert(number >= LONG_MIN && number <= LONG_MAX);
+  if (strict) {
+    assert(number >= LONG_MIN && number <= LONG_MAX);
+  } else if (!(number >= LONG_MIN && number <= LONG_MAX)) {
+    return def;
+  }
 #endif
   return (long)number;
 }
@@ -1787,9 +1788,12 @@ void vim_str2nr(const char_u *const start, int *const prep, int *const len,
 #define PARSE_NUMBER(base, cond, conv) \
   do { \
     while (!STRING_ENDED(ptr) && (cond)) { \
+      const uvarnumber_T digit = (uvarnumber_T)(conv); \
       /* avoid ubsan error for overflow */ \
-      if (un < UVARNUMBER_MAX / base) { \
-        un = base * un + (uvarnumber_T)(conv); \
+      if (un < UVARNUMBER_MAX / base \
+          || (un == UVARNUMBER_MAX / base \
+              && (base != 10 || digit <= UVARNUMBER_MAX % 10))) { \
+        un = base * un + digit; \
       } else { \
         un = UVARNUMBER_MAX; \
       } \
@@ -1906,7 +1910,8 @@ void backslash_halve(char_u *p)
 /// @param p
 ///
 /// @return String with the number of backslashes halved.
-char_u* backslash_halve_save(char_u *p)
+char_u *backslash_halve_save(const char_u *p)
+  FUNC_ATTR_NONNULL_ALL FUNC_ATTR_NONNULL_RET
 {
   // TODO(philix): simplify and improve backslash_halve_save algorithm
   char_u *res = vim_strsave(p);
