@@ -7,7 +7,6 @@
 
 #include <assert.h>
 #include <stdbool.h>
-#include <stdint.h>
 #include <inttypes.h>
 
 #include "nvim/vim.h"
@@ -18,6 +17,7 @@
 #include "nvim/ex_docmd.h"
 #include "nvim/ex_getln.h"
 #include "nvim/getchar.h"
+#include "nvim/misc1.h"
 #include "nvim/mbyte.h"
 #include "nvim/message.h"
 #include "nvim/memory.h"
@@ -1366,57 +1366,57 @@ static digr_T digraphdefault[] =
   { 'y', '`', 0x1ef3 },
 
   // Vim 5.x compatible digraphs that don't conflict with the above
-  { '~', '!', 161 },  // 
-  { 'c', '|', 162 },  // 
-  { '$', '$', 163 },  // 
-  { 'o', 'x', 164 },  //  - currency symbol in ISO 8859-1
-  { 'Y', '-', 165 },  // 
-  { '|', '|', 166 },  // 
-  { 'c', 'O', 169 },  // 
-  { '-', ',', 172 },  // 
-  { '-', '=', 175 },  // 
-  { '~', 'o', 176 },  // 
-  { '2', '2', 178 },  // 
-  { '3', '3', 179 },  // 
-  { 'p', 'p', 182 },  // 
-  { '~', '.', 183 },  // 
-  { '1', '1', 185 },  // 
-  { '~', '?', 191 },  // 
-  { 'A', '`', 192 },  // 
-  { 'A', '^', 194 },  // 
-  { 'A', '~', 195 },  // 
-  { 'A', '"', 196 },  // 
-  { 'A', '@', 197 },  // 
-  { 'E', '`', 200 },  // 
-  { 'E', '^', 202 },  // 
-  { 'E', '"', 203 },  // 
-  { 'I', '`', 204 },  // 
-  { 'I', '^', 206 },  // 
-  { 'I', '"', 207 },  // 
-  { 'N', '~', 209 },  // 
-  { 'O', '`', 210 },  // 
-  { 'O', '^', 212 },  // 
-  { 'O', '~', 213 },  // 
-  { '/', '\\', 215 },  //  - multiplication symbol in ISO 8859-1
-  { 'U', '`', 217 },  // 
-  { 'U', '^', 219 },  // 
-  { 'I', 'p', 222 },  // 
-  { 'a', '`', 224 },  // 
-  { 'a', '^', 226 },  // 
-  { 'a', '~', 227 },  // 
-  { 'a', '"', 228 },  // 
-  { 'a', '@', 229 },  // 
-  { 'e', '`', 232 },  // 
-  { 'e', '^', 234 },  // 
-  { 'e', '"', 235 },  // 
-  { 'i', '`', 236 },  // 
-  { 'i', '^', 238 },  // 
-  { 'n', '~', 241 },  // 
-  { 'o', '`', 242 },  // 
-  { 'o', '^', 244 },  // 
-  { 'o', '~', 245 },  // 
-  { 'u', '`', 249 },  // 
-  { 'u', '^', 251 },  // 
+  { '~', '!', 161 },  // ¡
+  { 'c', '|', 162 },  // ¢
+  { '$', '$', 163 },  // £
+  { 'o', 'x', 164 },  // ¤ - currency symbol in ISO 8859-1
+  { 'Y', '-', 165 },  // ¥
+  { '|', '|', 166 },  // ¦
+  { 'c', 'O', 169 },  // ©
+  { '-', ',', 172 },  // ¬
+  { '-', '=', 175 },  // ¯
+  { '~', 'o', 176 },  // °
+  { '2', '2', 178 },  // ²
+  { '3', '3', 179 },  // ³
+  { 'p', 'p', 182 },  // ¶
+  { '~', '.', 183 },  // ·
+  { '1', '1', 185 },  // ¹
+  { '~', '?', 191 },  // ¿
+  { 'A', '`', 192 },  // À
+  { 'A', '^', 194 },  // Â
+  { 'A', '~', 195 },  // Ã
+  { 'A', '"', 196 },  // Ä
+  { 'A', '@', 197 },  // Å
+  { 'E', '`', 200 },  // È
+  { 'E', '^', 202 },  // Ê
+  { 'E', '"', 203 },  // Ë
+  { 'I', '`', 204 },  // Ì
+  { 'I', '^', 206 },  // Î
+  { 'I', '"', 207 },  // Ï
+  { 'N', '~', 209 },  // Ñ
+  { 'O', '`', 210 },  // Ò
+  { 'O', '^', 212 },  // Ô
+  { 'O', '~', 213 },  // Õ
+  { '/', '\\', 215 },  // × - multiplication symbol in ISO 8859-1
+  { 'U', '`', 217 },  // Ù
+  { 'U', '^', 219 },  // Û
+  { 'I', 'p', 222 },  // Þ
+  { 'a', '`', 224 },  // à
+  { 'a', '^', 226 },  // â
+  { 'a', '~', 227 },  // ã
+  { 'a', '"', 228 },  // ä
+  { 'a', '@', 229 },  // å
+  { 'e', '`', 232 },  // è
+  { 'e', '^', 234 },  // ê
+  { 'e', '"', 235 },  // ë
+  { 'i', '`', 236 },  // ì
+  { 'i', '^', 238 },  // î
+  { 'n', '~', 241 },  // ñ
+  { 'o', '`', 242 },  // ò
+  { 'o', '^', 244 },  // ô
+  { 'o', '~', 245 },  // õ
+  { 'u', '`', 249 },  // ù
+  { 'u', '^', 251 },  // û
   { 'y', '"', 255 },  // x XX
 
   { NUL, NUL, NUL }
@@ -1496,8 +1496,8 @@ int get_digraph(int cmdline)
     }
 
     if (cmdline) {
-      if ((char2cells(c) == 1) && (cmdline_star == 0)) {
-        putcmdline(c, TRUE);
+      if ((char2cells(c) == 1) && c < 128 && (cmdline_star == 0)) {
+        putcmdline((char)c, true);
       }
     } else {
       add_to_showcmd(c);
@@ -1618,7 +1618,7 @@ void putdigraph(char_u *str)
       EMSG(_(e_number_exp));
       return;
     }
-    int n = getdigits_int(&str);
+    int n = getdigits_int(&str, true, 0);
 
     // If the digraph already exists, replace the result.
     dp = (digr_T *)user_digraphs.ga_data;
@@ -1663,13 +1663,13 @@ void listdigraphs(void)
       printdigraph(&tmp);
     }
     dp++;
-    os_breakcheck();
+    fast_breakcheck();
   }
 
   dp = (digr_T *)user_digraphs.ga_data;
   for (int i = 0; i < user_digraphs.ga_len && !got_int; ++i) {
     printdigraph(dp);
-    os_breakcheck();
+    fast_breakcheck();
     dp++;
   }
 }
