@@ -991,7 +991,7 @@ const char *tv_list_find_str(list_T *const l, const int n)
 {
   const listitem_T *const li = tv_list_find(l, n);
   if (li == NULL) {
-    emsgf(_(e_listidx), (int64_t)n);
+    EMSG2(_(e_listidx), (int64_t)n);
     return NULL;
   }
   return tv_get_string(TV_LIST_ITEM_TV(li));
@@ -1221,7 +1221,8 @@ void tv_dict_watcher_notify(dict_T *const dict, const char *const key,
 
 /// Allocate a dictionary item
 ///
-/// @note that the value of the item (->di_tv) still needs to be initialized.
+/// @note that the type and value of the item (->di_tv) still needs to
+///       be initialized.
 ///
 /// @param[in]  key  Key, is copied to the new item.
 /// @param[in]  key_len  Key length.
@@ -1235,12 +1236,14 @@ dictitem_T *tv_dict_item_alloc_len(const char *const key, const size_t key_len)
   memcpy(di->di_key, key, key_len);
   di->di_key[key_len] = NUL;
   di->di_flags = DI_FLAGS_ALLOC;
+  di->di_tv.v_lock = VAR_UNLOCKED;
   return di;
 }
 
 /// Allocate a dictionary item
 ///
-/// @note that the value of the item (->di_tv) still needs to be initialized.
+/// @note that the type and value of the item (->di_tv) still needs to
+///       be initialized.
 ///
 /// @param[in]  key  Key, is copied to the new item.
 ///
@@ -1269,7 +1272,7 @@ void tv_dict_item_free(dictitem_T *const item)
 /// @param[in]  di  Item to copy.
 ///
 /// @return [allocated] new dictionary item.
-static dictitem_T *tv_dict_item_copy(dictitem_T *const di)
+dictitem_T *tv_dict_item_copy(dictitem_T *const di)
   FUNC_ATTR_NONNULL_RET FUNC_ATTR_NONNULL_ALL FUNC_ATTR_WARN_UNUSED_RESULT
 {
   dictitem_T *const new_di = tv_dict_item_alloc((const char *)di->di_key);
@@ -1532,7 +1535,7 @@ bool tv_dict_get_callback(dict_T *const d,
   }
 
   if (!tv_is_func(di->di_tv) && di->di_tv.v_type != VAR_STRING) {
-    emsgf(_("E6000: Argument is not a function or function name"));
+    EMSG(_("E6000: Argument is not a function or function name"));
     return false;
   }
 
@@ -1572,7 +1575,6 @@ int tv_dict_add_list(dict_T *const d, const char *const key,
 {
   dictitem_T *const item = tv_dict_item_alloc_len(key, key_len);
 
-  item->di_tv.v_lock = VAR_UNLOCKED;
   item->di_tv.v_type = VAR_LIST;
   item->di_tv.vval.v_list = list;
   tv_list_ref(list);
@@ -1597,7 +1599,6 @@ int tv_dict_add_dict(dict_T *const d, const char *const key,
 {
   dictitem_T *const item = tv_dict_item_alloc_len(key, key_len);
 
-  item->di_tv.v_lock = VAR_UNLOCKED;
   item->di_tv.v_type = VAR_DICT;
   item->di_tv.vval.v_dict = dict;
   dict->dv_refcount++;
@@ -1621,7 +1622,6 @@ int tv_dict_add_nr(dict_T *const d, const char *const key,
 {
   dictitem_T *const item = tv_dict_item_alloc_len(key, key_len);
 
-  item->di_tv.v_lock = VAR_UNLOCKED;
   item->di_tv.v_type = VAR_NUMBER;
   item->di_tv.vval.v_number = nr;
   if (tv_dict_add(d, item) == FAIL) {
@@ -1644,7 +1644,6 @@ int tv_dict_add_special(dict_T *const d, const char *const key,
 {
   dictitem_T *const item = tv_dict_item_alloc_len(key, key_len);
 
-  item->di_tv.v_lock = VAR_UNLOCKED;
   item->di_tv.v_type = VAR_SPECIAL;
   item->di_tv.vval.v_special = val;
   if (tv_dict_add(d, item) == FAIL) {
@@ -1656,12 +1655,7 @@ int tv_dict_add_special(dict_T *const d, const char *const key,
 
 /// Add a string entry to dictionary
 ///
-/// @param[out]  d  Dictionary to add entry to.
-/// @param[in]  key  Key to add.
-/// @param[in]  key_len  Key length.
-/// @param[in]  val  String to add.
-///
-/// @return OK in case of success, FAIL when key already exists.
+/// @see tv_dict_add_allocated_str
 int tv_dict_add_str(dict_T *const d,
                     const char *const key, const size_t key_len,
                     const char *const val)
@@ -1670,6 +1664,27 @@ int tv_dict_add_str(dict_T *const d,
   return tv_dict_add_allocated_str(d, key, key_len, xstrdup(val));
 }
 
+/// Add a string entry to dictionary
+///
+/// @param[out]  d  Dictionary to add entry to.
+/// @param[in]  key  Key to add.
+/// @param[in]  key_len  Key length.
+/// @param[in]  val  String to add. NULL adds empty string.
+/// @param[in]  len  Use this many bytes from `val`, or -1 for whole string.
+///
+/// @return OK in case of success, FAIL when key already exists.
+int tv_dict_add_str_len(dict_T *const d,
+                        const char *const key, const size_t key_len,
+                        char *const val, int len)
+  FUNC_ATTR_NONNULL_ARG(1, 2)
+{
+  char *s = val ? val : "";
+  if (val != NULL) {
+    s = (len < 0) ? xstrdup(val) : xstrndup(val, (size_t)len);
+  }
+  return tv_dict_add_allocated_str(d, key, key_len, s);
+}
+
 /// Add a string entry to dictionary
 ///
 /// Unlike tv_dict_add_str() saves val to the new dictionary item in place of
@@ -1690,7 +1705,6 @@ int tv_dict_add_allocated_str(dict_T *const d,
 {
   dictitem_T *const item = tv_dict_item_alloc_len(key, key_len);
 
-  item->di_tv.v_lock = VAR_UNLOCKED;
   item->di_tv.v_type = VAR_STRING;
   item->di_tv.vval.v_string = (char_u *)val;
   if (tv_dict_add(d, item) == FAIL) {
@@ -2298,7 +2312,7 @@ void tv_item_lock(typval_T *const tv, const int deep, const bool lock)
   static int recurse = 0;
 
   if (recurse >= DICT_MAXNEST) {
-    emsgf(_("E743: variable nested too deep for (un)lock"));
+    EMSG(_("E743: variable nested too deep for (un)lock"));
     return;
   }
   if (deep == 0) {
@@ -2546,28 +2560,28 @@ bool tv_check_str_or_nr(const typval_T *const tv)
       return true;
     }
     case VAR_FLOAT: {
-      emsgf(_("E805: Expected a Number or a String, Float found"));
+      EMSG(_("E805: Expected a Number or a String, Float found"));
       return false;
     }
     case VAR_PARTIAL:
     case VAR_FUNC: {
-      emsgf(_("E703: Expected a Number or a String, Funcref found"));
+      EMSG(_("E703: Expected a Number or a String, Funcref found"));
       return false;
     }
     case VAR_LIST: {
-      emsgf(_("E745: Expected a Number or a String, List found"));
+      EMSG(_("E745: Expected a Number or a String, List found"));
       return false;
     }
     case VAR_DICT: {
-      emsgf(_("E728: Expected a Number or a String, Dictionary found"));
+      EMSG(_("E728: Expected a Number or a String, Dictionary found"));
       return false;
     }
     case VAR_SPECIAL: {
-      emsgf(_("E5300: Expected a Number or a String"));
+      EMSG(_("E5300: Expected a Number or a String"));
       return false;
     }
     case VAR_UNKNOWN: {
-      emsgf(_(e_intern2), "tv_check_str_or_nr(UNKNOWN)");
+      EMSG2(_(e_intern2), "tv_check_str_or_nr(UNKNOWN)");
       return false;
     }
   }
@@ -2611,7 +2625,7 @@ bool tv_check_num(const typval_T *const tv)
     case VAR_DICT:
     case VAR_FLOAT:
     case VAR_UNKNOWN: {
-      emsgf(_(num_errors[tv->v_type]));
+      EMSG(_(num_errors[tv->v_type]));
       return false;
     }
   }
@@ -2632,7 +2646,7 @@ static const char *const str_errors[] = {
 
 #undef FUNC_ERROR
 
-/// Check that given value is a string or can be converted to it
+/// Check that given value is a VimL String or can be "cast" to it.
 ///
 /// Error messages are compatible with tv_get_string_chk() previously used for
 /// the same purpose.
@@ -2655,7 +2669,7 @@ bool tv_check_str(const typval_T *const tv)
     case VAR_DICT:
     case VAR_FLOAT:
     case VAR_UNKNOWN: {
-      emsgf(_(str_errors[tv->v_type]));
+      EMSG(_(str_errors[tv->v_type]));
       return false;
     }
   }
@@ -2702,7 +2716,7 @@ varnumber_T tv_get_number_chk(const typval_T *const tv, bool *const ret_error)
     case VAR_LIST:
     case VAR_DICT:
     case VAR_FLOAT: {
-      emsgf(_(num_errors[tv->v_type]));
+      EMSG(_(num_errors[tv->v_type]));
       break;
     }
     case VAR_NUMBER: {
@@ -2778,23 +2792,23 @@ float_T tv_get_float(const typval_T *const tv)
     }
     case VAR_PARTIAL:
     case VAR_FUNC: {
-      emsgf(_("E891: Using a Funcref as a Float"));
+      EMSG(_("E891: Using a Funcref as a Float"));
       break;
     }
     case VAR_STRING: {
-      emsgf(_("E892: Using a String as a Float"));
+      EMSG(_("E892: Using a String as a Float"));
       break;
     }
     case VAR_LIST: {
-      emsgf(_("E893: Using a List as a Float"));
+      EMSG(_("E893: Using a List as a Float"));
       break;
     }
     case VAR_DICT: {
-      emsgf(_("E894: Using a Dictionary as a Float"));
+      EMSG(_("E894: Using a Dictionary as a Float"));
       break;
     }
     case VAR_SPECIAL: {
-      emsgf(_("E907: Using a special value as a Float"));
+      EMSG(_("E907: Using a special value as a Float"));
       break;
     }
     case VAR_UNKNOWN: {
@@ -2805,7 +2819,7 @@ float_T tv_get_float(const typval_T *const tv)
   return 0;
 }
 
-/// Get the string value of a VimL object
+/// Get the string value of a "stringish" VimL object.
 ///
 /// @param[in]  tv  Object to get value of.
 /// @param  buf  Buffer used to hold numbers and special variables converted to
@@ -2840,17 +2854,17 @@ const char *tv_get_string_buf_chk(const typval_T *const tv, char *const buf)
     case VAR_DICT:
     case VAR_FLOAT:
     case VAR_UNKNOWN: {
-      emsgf(_(str_errors[tv->v_type]));
+      EMSG(_(str_errors[tv->v_type]));
       return false;
     }
   }
   return NULL;
 }
 
-/// Get the string value of a VimL object
+/// Get the string value of a "stringish" VimL object.
 ///
 /// @warning For number and special values it uses a single, static buffer. It
-///          may be used only once, next call to get_tv_string may reuse it. Use
+///          may be used only once, next call to tv_get_string may reuse it. Use
 ///          tv_get_string_buf() if you need to use tv_get_string() output after
 ///          calling it again.
 ///
@@ -2866,10 +2880,10 @@ const char *tv_get_string_chk(const typval_T *const tv)
   return tv_get_string_buf_chk(tv, mybuf);
 }
 
-/// Get the string value of a VimL object
+/// Get the string value of a "stringish" VimL object.
 ///
 /// @warning For number and special values it uses a single, static buffer. It
-///          may be used only once, next call to get_tv_string may reuse it. Use
+///          may be used only once, next call to tv_get_string may reuse it. Use
 ///          tv_get_string_buf() if you need to use tv_get_string() output after
 ///          calling it again.
 ///
@@ -2888,7 +2902,7 @@ const char *tv_get_string(const typval_T *const tv)
   return tv_get_string_buf((typval_T *)tv, mybuf);
 }
 
-/// Get the string value of a VimL object
+/// Get the string value of a "stringish" VimL object.
 ///
 /// @note tv_get_string_chk() and tv_get_string_buf_chk() are similar, but
 ///       return NULL on error.
