@@ -4,8 +4,10 @@
 #include <stdbool.h>
 
 #include "nvim/os/time.h"
-#include "nvim/eval_defs.h"
 #include "nvim/pos.h"
+#include "nvim/eval/typval.h"
+#include "nvim/buffer_defs.h"
+#include "nvim/ex_cmds_defs.h"
 
 // flags for do_ecmd()
 #define ECMD_HIDE            0x01    // don't free the current buffer
