@@ -28,15 +28,15 @@ local FILES      = bit.bor(XFILE, EXTRA)
 local WORD1      = bit.bor(EXTRA, NOSPC)
 local FILE1      = bit.bor(FILES, NOSPC)
 
-local ADDR_LINES            =  0
-local ADDR_WINDOWS          =  1
-local ADDR_ARGUMENTS        =  2
-local ADDR_LOADED_BUFFERS   =  3
-local ADDR_BUFFERS          =  4
-local ADDR_TABS             =  5
-local ADDR_TABS_RELATIVE    =  6
-local ADDR_QUICKFIX         =  7
-local ADDR_OTHER            =  99
+local ADDR_LINES            =  0    -- buffer line numbers
+local ADDR_WINDOWS          =  1    -- window number
+local ADDR_ARGUMENTS        =  2    -- argument number
+local ADDR_LOADED_BUFFERS   =  3    -- buffer number of loaded buffer
+local ADDR_BUFFERS          =  4    -- buffer number
+local ADDR_TABS             =  5    -- tab page number
+local ADDR_TABS_RELATIVE    =  6    -- Tab page that only relative
+local ADDR_QUICKFIX         =  7    -- quickfix list entry number
+local ADDR_OTHER            =  99   -- something else
 
 -- The following table is described in ex_cmds_defs.h file.
 return {
@@ -108,7 +108,7 @@ return {
   },
   {
     command='argedit',
-    flags=bit.bor(BANG, NEEDARG, RANGE, NOTADR, ZEROR, FILE1, EDITCMD, ARGOPT, TRLBAR),
+    flags=bit.bor(BANG, NEEDARG, RANGE, NOTADR, ZEROR, FILES, EDITCMD, ARGOPT, TRLBAR),
     addr_type=ADDR_ARGUMENTS,
     func='ex_argedit',
   },
@@ -600,6 +600,12 @@ return {
     addr_type=ADDR_LINES,
     func='ex_wrongmodifier',
   },
+  {
+    command='const',
+    flags=bit.bor(NEEDARG, EXTRA, NOTRLCOM, CMDWIN),
+    addr_type=ADDR_LINES,
+    func='ex_const',
+  },
   {
     command='copen',
     flags=bit.bor(RANGE, NOTADR, COUNT, TRLBAR),
@@ -692,13 +698,13 @@ return {
   },
   {
     command='delcommand',
-    flags=bit.bor(NEEDARG, WORD1, TRLBAR, CMDWIN),
+    flags=bit.bor(BANG, NEEDARG, WORD1, TRLBAR, CMDWIN),
     addr_type=ADDR_LINES,
     func='ex_delcommand',
   },
   {
     command='delfunction',
-    flags=bit.bor(NEEDARG, WORD1, CMDWIN),
+    flags=bit.bor(BANG, NEEDARG, WORD1, CMDWIN),
     addr_type=ADDR_LINES,
     func='ex_delfunction',
   },
@@ -1004,7 +1010,7 @@ return {
   },
   {
     command='function',
-    flags=bit.bor(EXTRA, BANG, CMDWIN),
+    flags=bit.bor(EXTRA, BANG, SBOXOK, CMDWIN),
     addr_type=ADDR_LINES,
     func='ex_function',
   },
@@ -1706,24 +1712,6 @@ return {
     addr_type=ADDR_LINES,
     func='ex_next',
   },
-  {
-    command='nbkey',
-    flags=bit.bor(EXTRA, NOTADR, NEEDARG),
-    addr_type=ADDR_LINES,
-    func='ex_ni',
-  },
-  {
-    command='nbclose',
-    flags=bit.bor(TRLBAR, CMDWIN),
-    addr_type=ADDR_LINES,
-    func='ex_ni',
-  },
-  {
-    command='nbstart',
-    flags=bit.bor(WORD1, TRLBAR, CMDWIN),
-    addr_type=ADDR_LINES,
-    func='ex_ni',
-  },
   {
     command='new',
     flags=bit.bor(BANG, FILE1, RANGE, NOTADR, EDITCMD, ARGOPT, TRLBAR),
@@ -1958,18 +1946,6 @@ return {
     addr_type=ADDR_LINES,
     func='ex_previous',
   },
-  {
-    command='promptfind',
-    flags=bit.bor(EXTRA, NOTRLCOM, CMDWIN),
-    addr_type=ADDR_LINES,
-    func='ex_ni',
-  },
-  {
-    command='promptrepl',
-    flags=bit.bor(EXTRA, NOTRLCOM, CMDWIN),
-    addr_type=ADDR_LINES,
-    func='ex_ni',
-  },
   {
     command='profile',
     flags=bit.bor(BANG, EXTRA, TRLBAR, CMDWIN),
@@ -2096,6 +2072,30 @@ return {
     addr_type=ADDR_LINES,
     func='ex_py3file',
   },
+  {
+    command='pyx',
+    flags=bit.bor(RANGE, EXTRA, NEEDARG, CMDWIN),
+    addr_type=ADDR_LINES,
+    func='ex_pyx',
+  },
+  {
+    command='pyxdo',
+    flags=bit.bor(RANGE, DFLALL, EXTRA, NEEDARG, CMDWIN),
+    addr_type=ADDR_LINES,
+    func='ex_pyxdo',
+  },
+  {
+    command='pythonx',
+    flags=bit.bor(RANGE, EXTRA, NEEDARG, CMDWIN),
+    addr_type=ADDR_LINES,
+    func='ex_pyx',
+  },
+  {
+    command='pyxfile',
+    flags=bit.bor(RANGE, FILE1, NEEDARG, CMDWIN),
+    addr_type=ADDR_LINES,
+    func='ex_pyxfile',
+  },
   {
     command='quit',
     flags=bit.bor(BANG, RANGE, COUNT, NOTADR, TRLBAR, CMDWIN),
@@ -2150,6 +2150,12 @@ return {
     addr_type=ADDR_LINES,
     func='ex_redrawstatus',
   },
+  {
+    command='redrawtabline',
+    flags=bit.bor(TRLBAR, CMDWIN),
+    addr_type=ADDR_LINES,
+    func='ex_redrawtabline',
+  },
   {
     command='registers',
     flags=bit.bor(EXTRA, NOTRLCOM, TRLBAR, CMDWIN),
@@ -2158,7 +2164,7 @@ return {
   },
   {
     command='resize',
-    flags=bit.bor(RANGE, NOTADR, TRLBAR, WORD1),
+    flags=bit.bor(RANGE, NOTADR, TRLBAR, WORD1, CMDWIN),
     addr_type=ADDR_LINES,
     func='ex_resize',
   },
@@ -2326,8 +2332,8 @@ return {
   },
   {
     command='scriptnames',
-    flags=bit.bor(TRLBAR, CMDWIN),
-    addr_type=ADDR_LINES,
+    flags=bit.bor(BANG, RANGE, NOTADR, COUNT, TRLBAR, CMDWIN),
+    addr_type=ADDR_OTHER,
     func='ex_scriptnames',
   },
   {
@@ -2798,6 +2804,12 @@ return {
     addr_type=ADDR_LINES,
     func='ex_tag',
   },
+  {
+    command='tmenu',
+    flags=bit.bor(RANGE, NOTADR, ZEROR, EXTRA, TRLBAR, NOTRLCOM, USECTRLV, CMDWIN),
+    addr_type=ADDR_LINES,
+    func='ex_menu',
+  },
   {
     command='tmap',
     flags=bit.bor(EXTRA, TRLBAR, NOTRLCOM, USECTRLV, CMDWIN),
@@ -2810,12 +2822,6 @@ return {
     addr_type=ADDR_LINES,
     func='ex_mapclear',
   },
-  {
-    command='tmenu',
-    flags=bit.bor(RANGE, NOTADR, ZEROR, EXTRA, TRLBAR, NOTRLCOM, USECTRLV, CMDWIN),
-    addr_type=ADDR_LINES,
-    func='ex_menu',
-  },
   {
     command='tnext',
     flags=bit.bor(RANGE, NOTADR, BANG, TRLBAR, ZEROR),
@@ -2858,18 +2864,18 @@ return {
     addr_type=ADDR_LINES,
     func='ex_tag',
   },
-  {
-    command='tunmap',
-    flags=bit.bor(EXTRA, TRLBAR, NOTRLCOM, USECTRLV, CMDWIN),
-    addr_type=ADDR_LINES,
-    func='ex_unmap',
-  },
   {
     command='tunmenu',
     flags=bit.bor(EXTRA, TRLBAR, NOTRLCOM, USECTRLV, CMDWIN),
     addr_type=ADDR_LINES,
     func='ex_menu',
   },
+  {
+    command='tunmap',
+    flags=bit.bor(EXTRA, TRLBAR, NOTRLCOM, USECTRLV, CMDWIN),
+    addr_type=ADDR_LINES,
+    func='ex_unmap',
+  },
   {
     command='undo',
     flags=bit.bor(RANGE, NOTADR, COUNT, ZEROR, TRLBAR, CMDWIN),
@@ -3076,13 +3082,13 @@ return {
   },
   {
     command='wincmd',
-    flags=bit.bor(NEEDARG, WORD1, RANGE, NOTADR),
+    flags=bit.bor(NEEDARG, WORD1, RANGE, NOTADR, CMDWIN),
     addr_type=ADDR_WINDOWS,
     func='ex_wincmd',
   },
   {
     command='windo',
-    flags=bit.bor(BANG, NEEDARG, EXTRA, NOTRLCOM, RANGE, NOTADR, DFLALL),
+    flags=bit.bor(NEEDARG, EXTRA, NOTRLCOM, RANGE, NOTADR, DFLALL),
     addr_type=ADDR_WINDOWS,
     func='ex_listdo',
   },
@@ -3116,12 +3122,6 @@ return {
     addr_type=ADDR_LINES,
     func='do_wqall',
   },
-  {
-    command='wsverb',
-    flags=bit.bor(EXTRA, NOTADR, NEEDARG),
-    addr_type=ADDR_LINES,
-    func='ex_ni',
-  },
   {
     command='wshada',
     flags=bit.bor(BANG, FILE1, TRLBAR, CMDWIN),
