@@ -11,6 +11,11 @@
 #  define N_(x) x
 # endif
 # define NGETTEXT(x, xs, n) ngettext(x, xs, n)
+// On a Mac, gettext's libintl.h defines "setlocale" to be replaced by
+// "libintl_setlocal" which leads to wrong return values. #9789
+# if defined(__APPLE__) && defined(setlocale)
+#  undef setlocale
+# endif
 #else
 # define _(x) ((char *)(x))
 # define N_(x) x
