@@ -12,6 +12,7 @@
 #include "nvim/vim.h"
 #include "nvim/api/private/helpers.h"
 #include "nvim/ascii.h"
+#include "nvim/eval/typval.h"
 #include "nvim/popupmnu.h"
 #include "nvim/charset.h"
 #include "nvim/ex_cmds.h"
@@ -19,6 +20,7 @@
 #include "nvim/move.h"
 #include "nvim/option.h"
 #include "nvim/screen.h"
+#include "nvim/ui_compositor.h"
 #include "nvim/search.h"
 #include "nvim/strings.h"
 #include "nvim/memory.h"
@@ -35,13 +37,18 @@ static int pum_height;              // nr of displayed pum items
 static int pum_width;               // width of displayed pum items
 static int pum_base_width;          // width of pum items base
 static int pum_kind_width;          // width of pum items kind column
+static int pum_extra_width;         // width of extra stuff
 static int pum_scrollbar;           // TRUE when scrollbar present
 
+static int pum_anchor_grid;         // grid where position is defined
 static int pum_row;                 // top row of pum
 static int pum_col;                 // left column of pum
+static bool pum_above;              // pum is drawn above cursor line
 
 static bool pum_is_visible = false;
+static bool pum_is_drawn = false;
 static bool pum_external = false;
+static bool pum_invalid = false;  // the screen was just cleared
 
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "popupmnu.c.generated.h"
@@ -49,6 +56,32 @@ static bool pum_external = false;
 #define PUM_DEF_HEIGHT 10
 #define PUM_DEF_WIDTH  15
 
+static void pum_compute_size(void)
+{
+  // Compute the width of the widest match and the widest extra.
+  pum_base_width = 0;
+  pum_kind_width = 0;
+  pum_extra_width = 0;
+  for (int i = 0; i < pum_size; i++) {
+    int w = vim_strsize(pum_array[i].pum_text);
+    if (pum_base_width < w) {
+      pum_base_width = w;
+    }
+    if (pum_array[i].pum_kind != NULL) {
+      w = vim_strsize(pum_array[i].pum_kind) + 1;
+      if (pum_kind_width < w) {
+        pum_kind_width = w;
+      }
+    }
+    if (pum_array[i].pum_extra != NULL) {
+      w = vim_strsize(pum_array[i].pum_extra) + 1;
+      if (pum_extra_width < w) {
+        pum_extra_width = w;
+      }
+    }
+  }
+}
+
 /// Show the popup menu with items "array[size]".
 /// "array" must remain valid until pum_undisplay() is called!
 /// When possible the leftmost character is aligned with screen column "col".
@@ -60,47 +93,60 @@ static bool pum_external = false;
 /// @param array_changed if true, array contains different items since last call
 ///                      if false, a new item is selected, but the array
 ///                      is the same
-void pum_display(pumitem_T *array, int size, int selected, bool array_changed)
+/// @param cmd_startcol only for cmdline mode: column of completed match
+void pum_display(pumitem_T *array, int size, int selected, bool array_changed,
+                 int cmd_startcol)
 {
-  int w;
   int def_width;
-  int max_width;
-  int kind_width;
-  int extra_width;
-  int i;
-  int row;
   int context_lines;
-  int col;
   int above_row;
   int below_row;
   int redo_count = 0;
+  int row;
+  int col;
 
   if (!pum_is_visible) {
     // To keep the code simple, we only allow changing the
     // draw mode when the popup menu is not being displayed
-    pum_external = ui_is_external(kUIPopupmenu);
+    pum_external = ui_has(kUIPopupmenu)
+                   || (State == CMDLINE && ui_has(kUIWildmenu));
   }
 
   do {
     // Mark the pum as visible already here,
     // to avoid that must_redraw is set when 'cursorcolumn' is on.
     pum_is_visible = true;
+    pum_is_drawn = true;
     validate_cursor_col();
     above_row = 0;
     below_row = cmdline_row;
 
-    // anchor position: the start of the completed word
-    row = curwin->w_wrow + curwin->w_winrow;
-    if (curwin->w_p_rl) {
-      col = curwin->w_wincol + curwin->w_width - curwin->w_wcol - 1;
+    // wildoptions=pum
+    if (State == CMDLINE) {
+      row = ui_has(kUICmdline) ? 0 : cmdline_row;
+      col = cmd_startcol;
+      pum_anchor_grid = ui_has(kUICmdline) ? -1 : DEFAULT_GRID_HANDLE;
     } else {
-      col = curwin->w_wincol + curwin->w_wcol;
+      // anchor position: the start of the completed word
+      row = curwin->w_wrow;
+      if (curwin->w_p_rl) {
+        col = curwin->w_width - curwin->w_wcol - 1;
+      } else {
+        col = curwin->w_wcol;
+      }
+
+      pum_anchor_grid = (int)curwin->w_grid.handle;
+      if (!ui_has(kUIMultigrid)) {
+        pum_anchor_grid = (int)default_grid.handle;
+        row += curwin->w_winrow;
+        col += curwin->w_wincol;
+      }
     }
 
     if (pum_external) {
       if (array_changed) {
         Array arr = ARRAY_DICT_INIT;
-        for (i = 0; i < size; i++) {
+        for (int i = 0; i < size; i++) {
           Array item = ARRAY_DICT_INIT;
           ADD(item, STRING_OBJ(cstr_to_string((char *)array[i].pum_text)));
           ADD(item, STRING_OBJ(cstr_to_string((char *)array[i].pum_kind)));
@@ -108,17 +154,14 @@ void pum_display(pumitem_T *array, int size, int selected, bool array_changed)
           ADD(item, STRING_OBJ(cstr_to_string((char *)array[i].pum_info)));
           ADD(arr, ARRAY_OBJ(item));
         }
-        ui_call_popupmenu_show(arr, selected, row, col);
+        ui_call_popupmenu_show(arr, selected, row, col, pum_anchor_grid);
       } else {
         ui_call_popupmenu_select(selected);
+        return;
       }
-      return;
     }
 
     def_width = PUM_DEF_WIDTH;
-    max_width = 0;
-    kind_width = 0;
-    extra_width = 0;
 
     win_T *pvwin = NULL;
     FOR_ALL_WINDOWS_IN_TAB(wp, curtab) {
@@ -152,6 +195,7 @@ void pum_display(pumitem_T *array, int size, int selected, bool array_changed)
     if (row + 2 >= below_row - pum_height
         && row - above_row > (below_row - above_row) / 2) {
       // pum above "row"
+      pum_above = true;
 
       // Leave two lines of context if possible
       if (curwin->w_wrow - curwin->w_cline_row >= 2) {
@@ -174,6 +218,7 @@ void pum_display(pumitem_T *array, int size, int selected, bool array_changed)
       }
     } else {
       // pum below "row"
+      pum_above = false;
 
       // Leave two lines of context if possible
       if (curwin->w_cline_row + curwin->w_cline_height - curwin->w_wrow >= 3) {
@@ -211,33 +256,14 @@ void pum_display(pumitem_T *array, int size, int selected, bool array_changed)
         pum_height = row - above_row;
       }
     }
-
-    // Compute the width of the widest match and the widest extra.
-    for (i = 0; i < size; i++) {
-      w = vim_strsize(array[i].pum_text);
-
-      if (max_width < w) {
-        max_width = w;
-      }
-
-      if (array[i].pum_kind != NULL) {
-        w = vim_strsize(array[i].pum_kind) + 1;
-
-        if (kind_width < w) {
-          kind_width = w;
-        }
-      }
-
-      if (array[i].pum_extra != NULL) {
-        w = vim_strsize(array[i].pum_extra) + 1;
-
-        if (extra_width < w) {
-          extra_width = w;
-        }
-      }
+    if (pum_external) {
+      return;
     }
-    pum_base_width = max_width;
-    pum_kind_width = kind_width;
+
+    pum_array = array;
+    pum_size = size;
+    pum_compute_size();
+    int max_width = pum_base_width;
 
     // if there are more items than room we need a scrollbar
     if (pum_height < size) {
@@ -264,9 +290,9 @@ void pum_display(pumitem_T *array, int size, int selected, bool array_changed)
         pum_width = (int)(Columns - pum_col - pum_scrollbar);
       }
 
-      if ((pum_width > max_width + kind_width + extra_width + 1)
+      if ((pum_width > max_width + pum_kind_width + pum_extra_width + 1)
           && (pum_width > PUM_DEF_WIDTH)) {
-        pum_width = max_width + kind_width + extra_width + 1;
+        pum_width = max_width + pum_kind_width + pum_extra_width + 1;
 
         if (pum_width < PUM_DEF_WIDTH) {
           pum_width = PUM_DEF_WIDTH;
@@ -298,9 +324,6 @@ void pum_display(pumitem_T *array, int size, int selected, bool array_changed)
       pum_width = max_width - pum_scrollbar;
     }
 
-    pum_array = array;
-    pum_size = size;
-
     // Set selected item and redraw.  If the window size changed need to redo
     // the positioning.  Limit this to two times, when there is not much
     // room the window size will keep changing.
@@ -310,7 +333,7 @@ void pum_display(pumitem_T *array, int size, int selected, bool array_changed)
 /// Redraw the popup menu, using "pum_first" and "pum_selected".
 void pum_redraw(void)
 {
-  int row = pum_row;
+  int row = 0;
   int col;
   int attr_norm = win_hl_attr(curwin, HLF_PNI);
   int attr_select = win_hl_attr(curwin, HLF_PSI);
@@ -327,6 +350,47 @@ void pum_redraw(void)
   int round;
   int n;
 
+  int grid_width = pum_width;
+  int col_off = 0;
+  bool extra_space = false;
+  if (curwin->w_p_rl) {
+    col_off = pum_width;
+    if (pum_col < curwin->w_wincol + curwin->w_width - 1) {
+      grid_width += 1;
+      extra_space = true;
+    }
+  } else if (pum_col > 0) {
+    grid_width += 1;
+    col_off = 1;
+    extra_space = true;
+  }
+  if (pum_scrollbar > 0) {
+    grid_width++;
+  }
+
+  grid_assign_handle(&pum_grid);
+  bool moved = ui_comp_put_grid(&pum_grid, pum_row, pum_col-col_off,
+                                pum_height, grid_width, false, true);
+  bool invalid_grid = moved || pum_invalid;
+  pum_invalid = false;
+  must_redraw_pum = false;
+
+  if (!pum_grid.chars
+      || pum_grid.Rows != pum_height || pum_grid.Columns != grid_width) {
+    grid_alloc(&pum_grid, pum_height, grid_width, !invalid_grid, false);
+    ui_call_grid_resize(pum_grid.handle, pum_grid.Columns, pum_grid.Rows);
+  } else if (invalid_grid) {
+    grid_invalidate(&pum_grid);
+  }
+  if (ui_has(kUIMultigrid)) {
+    const char *anchor = pum_above ? "SW" : "NW";
+    int row_off = pum_above ? pum_height : 0;
+    ui_call_win_float_pos(pum_grid.handle, -1, cstr_to_string(anchor),
+                          pum_anchor_grid, pum_row-row_off, pum_col-col_off,
+                          false);
+  }
+
+
   // Never display more than we have
   if (pum_first > pum_size - pum_height) {
     pum_first = pum_size - pum_height;
@@ -346,20 +410,20 @@ void pum_redraw(void)
     idx = i + pum_first;
     attr = (idx == pum_selected) ? attr_select : attr_norm;
 
-    screen_puts_line_start(row);
+    grid_puts_line_start(&pum_grid, row);
 
     // prepend a space if there is room
-    if (curwin->w_p_rl) {
-      if (pum_col < curwin->w_wincol + curwin->w_width - 1) {
-        screen_putchar(' ', row, pum_col + 1, attr);
+    if (extra_space) {
+      if (curwin->w_p_rl) {
+        grid_putchar(&pum_grid, ' ', row, col_off + 1, attr);
+      } else {
+        grid_putchar(&pum_grid, ' ', row, col_off - 1, attr);
       }
-    } else if (pum_col > 0) {
-      screen_putchar(' ', row, pum_col - 1, attr);
     }
 
     // Display each entry, use two spaces for a Tab.
     // Do this 3 times: For the main text, kind and extra info
-    col = pum_col;
+    col = col_off;
     totwidth = 0;
 
     for (round = 1; round <= 3; ++round) {
@@ -416,12 +480,13 @@ void pum_redraw(void)
                   size++;
                 }
               }
-              screen_puts_len(rt, (int)STRLEN(rt), row, col - size + 1, attr);
+              grid_puts_len(&pum_grid, rt, (int)STRLEN(rt), row,
+                            col - size + 1, attr);
               xfree(rt_start);
               xfree(st);
               col -= width;
             } else {
-              screen_puts_len(st, (int)STRLEN(st), row, col, attr);
+              grid_puts_len(&pum_grid, st, (int)STRLEN(st), row, col, attr);
               xfree(st);
               col += width;
             }
@@ -432,10 +497,11 @@ void pum_redraw(void)
 
             // Display two spaces for a Tab.
             if (curwin->w_p_rl) {
-              screen_puts_len((char_u *)"  ", 2, row, col - 1, attr);
+              grid_puts_len(&pum_grid, (char_u *)"  ", 2, row, col - 1,
+                            attr);
               col -= 2;
             } else {
-              screen_puts_len((char_u *)"  ", 2, row, col, attr);
+              grid_puts_len(&pum_grid, (char_u *)"  ", 2, row, col, attr);
               col += 2;
             }
             totwidth += 2;
@@ -466,36 +532,37 @@ void pum_redraw(void)
       }
 
       if (curwin->w_p_rl) {
-        screen_fill(row, row + 1, pum_col - pum_base_width - n + 1,
-                    col + 1, ' ', ' ', attr);
-        col = pum_col - pum_base_width - n + 1;
+        grid_fill(&pum_grid, row, row + 1, col_off - pum_base_width - n + 1,
+                  col + 1, ' ', ' ', attr);
+        col = col_off - pum_base_width - n + 1;
       } else {
-        screen_fill(row, row + 1, col, pum_col + pum_base_width + n,
-                    ' ', ' ', attr);
-        col = pum_col + pum_base_width + n;
+        grid_fill(&pum_grid, row, row + 1, col,
+                  col_off + pum_base_width + n, ' ', ' ', attr);
+        col = col_off + pum_base_width + n;
       }
       totwidth = pum_base_width + n;
     }
 
     if (curwin->w_p_rl) {
-      screen_fill(row, row + 1, pum_col - pum_width + 1, col + 1, ' ', ' ',
-                  attr);
+      grid_fill(&pum_grid, row, row + 1, col_off - pum_width + 1, col + 1,
+                ' ', ' ', attr);
     } else {
-      screen_fill(row, row + 1, col, pum_col + pum_width, ' ', ' ', attr);
+      grid_fill(&pum_grid, row, row + 1, col, col_off + pum_width, ' ', ' ',
+                attr);
     }
 
     if (pum_scrollbar > 0) {
       if (curwin->w_p_rl) {
-        screen_putchar(' ', row, pum_col - pum_width,
-                       i >= thumb_pos && i < thumb_pos + thumb_heigth
-                       ? attr_thumb : attr_scroll);
+        grid_putchar(&pum_grid, ' ', row, col_off - pum_width,
+                     i >= thumb_pos && i < thumb_pos + thumb_heigth
+                     ? attr_thumb : attr_scroll);
       } else {
-        screen_putchar(' ', row, pum_col + pum_width,
-                       i >= thumb_pos && i < thumb_pos + thumb_heigth
-                       ? attr_thumb : attr_scroll);
+        grid_putchar(&pum_grid, ' ', row, col_off + pum_width,
+                     i >= thumb_pos && i < thumb_pos + thumb_heigth
+                     ? attr_thumb : attr_scroll);
       }
     }
-    screen_puts_line_flush(false);
+    grid_puts_line_flush(false);
     row++;
   }
 }
@@ -603,7 +670,7 @@ static int pum_set_selected(int n, int repeat)
             && (curbuf->b_p_bh[0] == 'w')) {
           // Already a "wipeout" buffer, make it empty.
           while (!BUFEMPTY()) {
-            ml_delete((linenr_T)1, FALSE);
+            ml_delete((linenr_T)1, false);
           }
         } else {
           // Don't want to sync undo in the current buffer.
@@ -628,11 +695,11 @@ static int pum_set_selected(int n, int repeat)
           for (p = pum_array[pum_selected].pum_info; *p != NUL;) {
             e = vim_strchr(p, '\n');
             if (e == NULL) {
-              ml_append(lnum++, p, 0, FALSE);
+              ml_append(lnum++, p, 0, false);
               break;
             } else {
               *e = NUL;
-              ml_append(lnum++, p, (int)(e - p + 1), FALSE);
+              ml_append(lnum++, p, (int)(e - p + 1), false);
               *e = '\n';
               p = e + 1;
             }
@@ -686,6 +753,8 @@ static int pum_set_selected(int n, int repeat)
 
             // Update the screen before drawing the popup menu.
             // Enable updating the status lines.
+            // TODO(bfredl): can simplify, get rid of the flag munging?
+            // or at least eliminate extra redraw before win_enter()?
             pum_is_visible = false;
             update_screen(0);
             pum_is_visible = true;
@@ -715,17 +784,32 @@ static int pum_set_selected(int n, int repeat)
 }
 
 /// Undisplay the popup menu (later).
-void pum_undisplay(void)
+void pum_undisplay(bool immediate)
 {
   pum_is_visible = false;
   pum_array = NULL;
+  must_redraw_pum = false;
 
-  if (pum_external) {
-    ui_call_popupmenu_hide();
-  } else {
-    redraw_all_later(SOME_VALID);
-    redraw_tabline = true;
-    status_redraw_all();
+  if (immediate) {
+    pum_check_clear();
+  }
+}
+
+void pum_check_clear(void)
+{
+  if (!pum_is_visible && pum_is_drawn) {
+    if (pum_external) {
+      ui_call_popupmenu_hide();
+    } else {
+      ui_comp_remove_grid(&pum_grid);
+      if (ui_has(kUIMultigrid)) {
+        ui_call_win_close(pum_grid.handle);
+        ui_call_grid_destroy(pum_grid.handle);
+      }
+      // TODO(bfredl): consider keeping float grids allocated.
+      grid_free(&pum_grid);
+    }
+    pum_is_drawn = false;
   }
 }
 
@@ -748,11 +832,43 @@ bool pum_drawn(void)
   return pum_visible() && !pum_external;
 }
 
+/// Screen was cleared, need to redraw next time
+void pum_invalidate(void)
+{
+  pum_invalid = true;
+}
+
+void pum_recompose(void)
+{
+  ui_comp_compose_grid(&pum_grid);
+}
+
 /// Gets the height of the menu.
 ///
 /// @return the height of the popup menu, the number of entries visible.
 /// Only valid when pum_visible() returns TRUE!
 int pum_get_height(void)
 {
+  if (pum_external) {
+    int ui_pum_height = ui_pum_get_height();
+    if (ui_pum_height) {
+      return ui_pum_height;
+    }
+  }
   return pum_height;
 }
+
+/// Add size information about the pum to "dict".
+void pum_set_event_info(dict_T *dict)
+{
+  if (!pum_visible()) {
+    return;
+  }
+  tv_dict_add_nr(dict, S_LEN("height"), pum_height);
+  tv_dict_add_nr(dict, S_LEN("width"), pum_width);
+  tv_dict_add_nr(dict, S_LEN("row"), pum_row);
+  tv_dict_add_nr(dict, S_LEN("col"), pum_col);
+  tv_dict_add_nr(dict, S_LEN("size"), pum_size);
+  tv_dict_add_special(dict, S_LEN("scrollbar"),
+                      pum_scrollbar ? kSpecialVarTrue : kSpecialVarFalse);
+}
