@@ -1,6 +1,9 @@
 #ifndef NVIM_QUICKFIX_H
 #define NVIM_QUICKFIX_H
 
+#include "nvim/types.h"
+#include "nvim/ex_cmds_defs.h"
+
 /* flags for skip_vimgrep_pat() */
 #define VGR_GLOBAL      1
 #define VGR_NOJUMP      2
