@@ -3,22 +3,10 @@
 
 #include <stdbool.h>
 
+#include "nvim/globals.h"
 #include "nvim/buffer_defs.h"
 #include "nvim/ex_cmds_defs.h"
 
-/// Terminal highlighting attribute bits.
-/// Attributes above HL_ALL are used for syntax highlighting.
-/// \addtogroup HL_ATTRIBUTES
-/// @{
-#define HL_NORMAL      0x00
-#define HL_INVERSE     0x01
-#define HL_BOLD        0x02
-#define HL_ITALIC      0x04
-#define HL_UNDERLINE   0x08
-#define HL_UNDERCURL   0x10
-#define HL_STANDOUT    0x20
-/// @}
-
 #define HL_CONTAINED   0x01    /* not used on toplevel */
 #define HL_TRANSP      0x02    /* has no highlighting	*/
 #define HL_ONELINE     0x04    /* match within one line only */
@@ -45,6 +33,9 @@ typedef struct {
 } color_name_table_T;
 extern color_name_table_T color_name_table[];
 
+/// Array of highlight definitions, used for unit testing
+extern const char *const highlight_init_cmdline[];
+
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "syntax.h.generated.h"
 #endif
