@@ -20,15 +20,15 @@ describe("update_menu notification", function()
   end)
 
   local function expect_sent(expected)
-    screen:wait(function()
+    screen:expect{condition=function()
       if screen.update_menu ~= expected then
         if expected then
-          return 'update_menu was expected but not sent'
+          error('update_menu was expected but not sent')
         else
-          return 'update_menu was sent unexpectedly'
+          error('update_menu was sent unexpectedly')
         end
       end
-    end)
+    end, unchanged=(not expected)}
   end
 
   it("should be sent when adding a menu", function()
