@@ -7,14 +7,14 @@ describe('TabNewEntered', function()
       it('matches when entering any new tab', function()
         clear()
         nvim('command', 'au! TabNewEntered * echom "tabnewentered:".tabpagenr().":".bufnr("")')
-        eq("\ntabnewentered:2:2", nvim('command_output', 'tabnew'))
-        eq("\n\"test.x2\" [New File]\ntabnewentered:3:3", nvim('command_output', 'tabnew test.x2'))
+        eq("tabnewentered:2:2", nvim('command_output', 'tabnew'))
+        eq("tabnewentered:3:3", nvim('command_output', 'tabnew test.x2'))
      end)
     end)
     describe('with FILE as <afile>', function()
       it('matches when opening a new tab for FILE', function()
         nvim('command', 'au! TabNewEntered Xtest-tabnewentered echom "tabnewentered:match"')
-        eq('\n"Xtest-tabnewentered" [New File]\ntabnewentered:4:4\ntabnewentered:match',
+        eq('tabnewentered:4:4\ntabnewentered:match',
           nvim('command_output', 'tabnew Xtest-tabnewentered'))
      end)
     end)
@@ -24,7 +24,7 @@ describe('TabNewEntered', function()
         nvim('command', 'au! TabNewEntered * echom "entered"')
         nvim('command', 'tabnew test.x2')
         nvim('command', 'split')
-        eq('\nentered', nvim('command_output', 'execute "normal \\<C-W>T"'))
+        eq('entered', nvim('command_output', 'execute "normal \\<C-W>T"'))
       end)
     end)
   end)
