@@ -18,7 +18,8 @@ describe('mf_hash_grow()', function()
   setup(clear)
 
   -- Check to see if cksum exists, otherwise skip the test
-  if os.execute('which cksum 2>&1 > /dev/null') ~= 0 then
+  local null = helpers.iswin() and 'nul' or '/dev/null'
+  if os.execute('cksum --help >' .. null .. ' 2>&1') ~= 0 then
     pending('was not tested because cksum was not found', function() end)
   else
     it('is working', function()
