@@ -2,7 +2,7 @@
 
 local helpers = require('test.functional.helpers')(after_each)
 local clear, feed, insert = helpers.clear, helpers.feed, helpers.insert
-local execute, expect = helpers.execute, helpers.expect
+local feed_command, expect = helpers.feed_command, helpers.expect
 
 describe('insertcount', function()
   setup(clear)
@@ -13,7 +13,7 @@ describe('insertcount', function()
       Second line
       Last line]])
 
-    execute('/Second')
+    feed_command('/Second')
     feed('4gro')
 
     expect([[
