@@ -5,6 +5,7 @@ local wait = helpers.wait
 local eval, feed_command, source = helpers.eval, helpers.feed_command, helpers.source
 local eq, neq = helpers.eq, helpers.neq
 local write_file = helpers.write_file
+local command= helpers.command
 
 describe(':terminal buffer', function()
   local screen
@@ -224,6 +225,22 @@ describe(':terminal buffer', function()
       neq('terminal', eval('&buftype'))
     end)
   end)
+
+  it('it works with set rightleft #11438', function()
+    local columns = eval('&columns')
+    feed(string.rep('a', columns))
+    command('set rightleft')
+    screen:expect([[
+                                               ydaer ytt|
+      {1:a}aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa|
+                                                        |
+                                                        |
+                                                        |
+                                                        |
+      {3:-- TERMINAL --}                                    |
+    ]])
+    command('bdelete!')
+  end)
 end)
 
 describe('No heap-buffer-overflow when using', function()
