@@ -26,7 +26,7 @@ describe('setqflist()', function()
   it('sets w:quickfix_title', function()
     setqflist({''}, 'r', 'foo')
     command('copen')
-    eq(':foo', get_cur_win_var('quickfix_title'))
+    eq('foo', get_cur_win_var('quickfix_title'))
     setqflist({''}, 'r', {['title'] = 'qf_title'})
     eq('qf_title', get_cur_win_var('quickfix_title'))
   end)
@@ -34,7 +34,7 @@ describe('setqflist()', function()
   it('allows string {what} for backwards compatibility', function()
     setqflist({}, 'r', '5')
     command('copen')
-    eq(':5', get_cur_win_var('quickfix_title'))
+    eq('5', get_cur_win_var('quickfix_title'))
   end)
 
   it('requires a dict for {what}', function()
@@ -64,8 +64,8 @@ describe('setloclist()', function()
     setloclist(1, {}, 'r', 'foo')
     setloclist(2, {}, 'r', 'bar')
     command('lopen')
-    eq(':bar', get_cur_win_var('quickfix_title'))
+    eq('bar', get_cur_win_var('quickfix_title'))
     command('lclose | wincmd w | lopen')
-    eq(':foo', get_cur_win_var('quickfix_title'))
+    eq('foo', get_cur_win_var('quickfix_title'))
   end)
 end)
