@@ -165,6 +165,7 @@ static const struct key_name_entry {
   { K_DEL,             "Del" },
   { K_DEL,             "Delete" },      // Alternative name
   { K_KDEL,            "kDel" },
+  { K_KDEL,            "KPPeriod" },    // termkey KPPeriod value
   { K_UP,              "Up" },
   { K_DOWN,            "Down" },
   { K_LEFT,            "Left" },
@@ -173,6 +174,14 @@ static const struct key_name_entry {
   { K_XDOWN,           "xDown" },
   { K_XLEFT,           "xLeft" },
   { K_XRIGHT,          "xRight" },
+  { K_KUP,             "kUp" },
+  { K_KUP,             "KP8" },
+  { K_KDOWN,           "kDown" },
+  { K_KDOWN,           "KP2" },
+  { K_KLEFT,           "kLeft" },
+  { K_KLEFT,           "KP4" },
+  { K_KRIGHT,          "kRight" },
+  { K_KRIGHT,          "KP6" },
 
   { K_F1,              "F1" },
   { K_F2,              "F2" },
@@ -225,8 +234,10 @@ static const struct key_name_entry {
   { K_INS,             "Insert" },
   { K_INS,             "Ins" },         // Alternative name
   { K_KINS,            "kInsert" },
+  { K_KINS,            "KP0" },
   { K_HOME,            "Home" },
   { K_KHOME,           "kHome" },
+  { K_KHOME,           "KP7" },
   { K_XHOME,           "xHome" },
   { K_ZHOME,           "zHome" },
   { K_END,             "End" },
@@ -236,13 +247,22 @@ static const struct key_name_entry {
   { K_PAGEUP,          "PageUp" },
   { K_PAGEDOWN,        "PageDown" },
   { K_KPAGEUP,         "kPageUp" },
+  { K_KPAGEUP,         "KP9" },
   { K_KPAGEDOWN,       "kPageDown" },
+  { K_KPAGEDOWN,       "KP3" },
+  { K_KORIGIN,         "kOrigin" },
+  { K_KORIGIN,         "KP5" },
 
   { K_KPLUS,           "kPlus" },
+  { K_KPLUS,           "KPPlus" },
   { K_KMINUS,          "kMinus" },
+  { K_KMINUS,          "KPMinus" },
   { K_KDIVIDE,         "kDivide" },
+  { K_KDIVIDE,         "KPDiv" },
   { K_KMULTIPLY,       "kMultiply" },
+  { K_KMULTIPLY,       "KPMult" },
   { K_KENTER,          "kEnter" },
+  { K_KENTER,          "KPEnter" },
   { K_KPOINT,          "kPoint" },
 
   { K_K0,              "k0" },
